Module: Procrastinator::Scheduler::SerialWorking
- Included in:
- WorkProxy
- Defined in:
- lib/procrastinator/scheduler.rb
Overview
Serial work style
Instance Method Summary collapse
-
#serially(steps: 1) ⇒ Object
Work off the given number of tasks for each queue and return.
Instance Method Details
#serially(steps: 1) ⇒ Object
Work off the given number of tasks for each queue and return
117 118 119 120 121 |
# File 'lib/procrastinator/scheduler.rb', line 117 def serially(steps: 1) steps.times do workers.each(&:work_one) end end |